local PortalMage = LibStub("AceAddon-3.0"):NewAddon("PortalMage")
local group = {}
local Masque
local teleportRunes
local portalRunes

portalMageSpells = {["stormwind"] = true,
					["orgrimmar"] = true,
					["ironforge"] = true,
					["undercity"] = true,
					["darnassus"] = true,
					["thunderbluff"] = true}

PortalMage.portals = {["stormwind"] = 10059,
					  ["orgrimmar"] = 11417,
					  ["ironforge"] = 11416,
					  ["undercity"] = 11418,
					  ["darnassus"] = 11419,
					  ["thunderbluff"] = 11420}

PortalMage.teleports = {["stormwind"] = 3561,
						["orgrimmar"] = 3567,
						["ironforge"] = 3562,
						["undercity"] = 3563,
						["darnassus"] = 3565,
						["thunderbluff"] = 3566}

PortalMage.icons = {["stormwind"] = "Interface/ICONS/Spell_Arcane_TeleportStormwind",
					["orgrimmar"] = "Interface/ICONS/Spell_Arcane_TeleportOrgrimmar",
					["ironforge"] = "Interface/ICONS/Spell_Arcane_TeleportIronforge",
					["undercity"] = "Interface/ICONS/Spell_Arcane_TeleportUndercity",
					["darnassus"] = "Interface/ICONS/Spell_Arcane_TeleportDarnassus",
					["thunderbluff"] = "Interface/ICONS/Spell_Arcane_TeleportThunderBluff"}

function PortalMage:OnInitialize()
	_, self.class = UnitClass("player")
	self.faction = UnitFactionGroup("player")
	if self.class == "MAGE" then
		self.frame = CreateFrame("FRAME", nil, UIParent)
		self.move = CreateFrame("FRAME", nil, self.frame)
		Masque = LibStub("Masque", true)
		if Masque ~= nil then
			group = Masque:Group("Portal Mage", "Buttons")
		end
		if not portalMageFrameX then
			portalMageFrameX = 300
		end
		if not portalMageFrameY then
			portalMageFrameY = 300
		end
		if portalMageVertical == nil then
			portalMageVertical = false
		end
		teleportRunes = self:CountItems(17031)
		portalRunes = self:CountItems(17032)
		self:SetupFrame(self.frame)
		self:SetupMoveFrame(self.move)
		if self.faction == "Alliance" then
			local options = {
				name = "Portal Mage",
				handler = PortalMage,
				type = "group",
				args = {
					movable = {
						name = "Set Movable",
						type = "toggle",
						desc = "makes bar movable",
						set = "SetMove",
						get = "GetMove"
					},
					layout = {
						name = "Layout",
						type = "group",
						args = {
							vertical = {
								name = "Vertical",
								type = "toggle",
								desc = "select layout type",
								set = "SetLayout",
								get = "GetLayout",
							}
						}
					},
					portals = {
						name = "Portals",
						type = "group",
						args = {
							stormwind = {
								name = "Stormwind",
								type = "toggle",
								desc = "toggles portal and teleport to Stormwind",
								set = "Set",
								get = "Get"
							},
							ironforge = {
								name = "Ironforge",
								type = "toggle",
								desc = "toggles portal and teleport to Ironforge",
								set = "Set",
								get = "Get"
							},
							darnassus = {
								name = "Darnassus",
								type = "toggle",
								desc = "toggles portal and teleport to Darnassus",
								set = "Set",
								get = "Get"
							}
						}
					}
				}
			}
			LibStub("AceConfig-3.0"):RegisterOptionsTable("PortalMage", options, nil)
			LibStub("AceConfigDialog-3.0"):AddToBlizOptions("PortalMage", "Portal Mage")
		else
			local options = {
				name = "Portal Mage",
				handler = PortalMage,
				type = "group",
				args = {
					movable = {
						name = "Set Movable",
						type = "toggle",
						desc = "makes bar movable",
						set = "SetMove",
						get = "GetMove"
					},
					layout = {
						name = "Layout",
						type = "group",
						args = {
							vertical = {
								name = "Vertical",
								type = "toggle",
								desc = "select layout type",
								set = "SetLayout",
								get = "GetLayout",
							}
						}
					},
					portals = {
						name = "Portals",
						type = "group",
						args = {
							orgrimmar = {
								name = "Orgrimmar",
								type = "toggle",
								desc = "toggles portal and teleport to Orgrimmar",
								set = "Set",
								get = "Get"
							},
							undercity = {
								name = "Undercity",
								type = "toggle",
								desc = "toggles portal and teleport to Undercity",
								set = "Set",
								get = "Get"
							},
							thunderbluff = {
								name = "Thunder Bluff",
								type = "toggle",
								desc = "toggles portal and teleport to Thunder Bluff",
								set = "Set",
								get = "Get"
							}
						}
					}
				}
			}
			LibStub("AceConfig-3.0"):RegisterOptionsTable("PortalMage", options, nil)
			LibStub("AceConfigDialog-3.0"):AddToBlizOptions("PortalMage", "Portal Mage")
		end
	end
end

function PortalMage:SetMove(info, val)
	if val then
		PortalMage.move:Show()
	else
		PortalMage.move:Hide()
	end
end

function PortalMage:GetMove(info)
	return PortalMage.move:IsShown()
end

function PortalMage:SetLayout(info, val)
	portalMageVertical = val
	if portalMageVertical then
		self:SetupButtonsVertical(self.frame, self.move)
	else
		self:SetupButtonsHorizontal(self.frame, self.move)
	end
end

function PortalMage:GetLayout(info)
	return portalMageVertical
end

function PortalMage:Set(info, val)
	portalMageSpells[info[#info]] = val
	if portalMageVertical then
		self:SetupButtonsVertical(self.frame, self.move)
	else
		self:SetupButtonsHorizontal(self.frame, self.move)
	end
	if Masque ~= nil then
		self:RemoveMasqueButtons()
		self:AddMasqueButtons()
		group:ReSkin()
	end
end

function PortalMage:Get(info)
	return portalMageSpells[info[#info]]
end

function PortalMage:AddMasqueButtons()
	if self.faction == "Alliance" then
		if portalMageSpells["stormwind"] then
			group:AddButton(self.button1)
		end
		if portalMageSpells["ironforge"] then
			group:AddButton(self.button2)
		end
		if portalMageSpells["darnassus"] then
			group:AddButton(self.button3)
		end
	else
		if portalMageSpells["orgrimmar"] then
			group:AddButton(self.button1)
		end
		if portalMageSpells["undercity"] then
			group:AddButton(self.button2)
		end
		if portalMageSpells["thunderbluff"] then
			group:AddButton(self.button3)
		end
	end
end

function PortalMage:RemoveMasqueButtons()
	group:RemoveButton(self.button1)
	group:RemoveButton(self.button2)
	group:RemoveButton(self.button3)
end

function PortalMage.frameOnEvent(self, event, spell)
	if event == "PLAYER_LOGIN" then
		self:SetHeight(40)
		PortalMage.move:SetHeight(40);
		if portalMageVertical then
			PortalMage:SetupButtonsVertical(self, PortalMage.move)
		else
			PortalMage:SetupButtonsHorizontal(self, PortalMage.move)
		end
		if Masque ~= nil then
			PortalMage:AddMasqueButtons()
			group:ReSkin()
		end
		self:UnregisterEvent("PLAYER_LOGIN")
	elseif event == "LEARNED_SPELL_IN_TAB" and PortalMage:NewSpell(spell) then
		self:SetHeight(40)
		PortalMage.move:SetHeight(40);
		if portalMageVertical then
			PortalMage:SetupButtonsVertical(self, PortalMage.move)
		else
			PortalMage:SetupButtonsHorizontal(self, PortalMage.move)
		end
		if Masque ~= nil then
			PortalMage:RemoveMasqueButtons()
			PortalMage:AddMasqueButtons()
			group:ReSkin()
		end
	elseif event == "BAG_UPDATE" then
		teleportRunes = PortalMage:CountItems(17031)
		portalRunes = PortalMage:CountItems(17032)
		self.teleports:SetText(teleportRunes)
		self.portals:SetText(portalRunes)
	end
end

function PortalMage.frameOnDragStart(self)
	PortalMage.frame:StartMoving()
end

function PortalMage.frameOnDragStop(self)
	PortalMage.frame:StopMovingOrSizing()
	portalMageFrameX = PortalMage.frame:GetLeft()
	portalMageFrameY = PortalMage.frame:GetBottom()
end

function PortalMage:NewSpell(id)
	if self:SpellOnList(id) then
		for i = 121,123,1 do
			_, spell = GetActionInfo(i)
			if spell == id then
				return false
			end
		end
		return true
	end
	return false
end

function PortalMage:SpellOnList(id)
	local name = GetSpellInfo(id)
	for k, v in pairs(self.portals) do
		if name == v then 
			return true
		end
	end
	return false
end

function PortalMage:SetupFrame(frame)
	frame:SetMovable(true)
	frame:SetPoint("BOTTOMLEFT", portalMageFrameX, portalMageFrameY)
	frame:SetScale(0.8)
	frame:SetFrameStrata("LOW")
	frame:RegisterEvent("PLAYER_LOGIN")
	frame:RegisterEvent("LEARNED_SPELL_IN_TAB")
	frame:RegisterEvent("BAG_UPDATE")
	frame:SetScript("OnEvent", PortalMage.frameOnEvent)
	frame.teleports = frame:CreateFontString(nil, "HIGH", "GameFontWhite")
	frame.teleports:SetTextColor(1, 1, 0)
	frame.teleports:SetText(teleportRunes)
	frame.portals = frame:CreateFontString(nil, "HIGH", "GameFontWhite")
	frame.portals:SetTextColor(0, 1, 0)
	frame.portals:SetText(portalRunes)
end

function PortalMage:SetupMoveFrame(frame)
	frame:SetMovable(true)
	frame:EnableMouse(true)
	frame:SetClampedToScreen(true)
	frame:SetPoint("BOTTOMLEFT", 0, 0)
	frame:SetFrameStrata("HIGH")
	local backdrop = {
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		tile = false,
		tileSize = 0,
	}
	frame:SetBackdrop(backdrop)
	frame:SetBackdropColor(0, 0.75, 0.25, 0.75)
	frame:RegisterForDrag("LeftButton")
	frame:SetScript("OnDragStart", PortalMage.frameOnDragStart)
	frame:SetScript("OnDragStop", PortalMage.frameOnDragStop)
	frame:Hide()
end

function PortalMage:CountItems(item)
	local count = 0
	for bag = 0 ,NUM_BAG_SLOTS do
		for slot = 1, GetContainerNumSlots(bag) do
			if item == GetContainerItemID(bag, slot) then
				count = count + (select(2, GetContainerItemInfo(bag, slot)))
			end
		end
	end
	return count
end

function PortalMage:SetupButtonsVertical(frame, move)
	self:ClearButtons()
	local length = 0
	if self.faction == "Alliance" then
		if portalMageSpells["stormwind"] then
			self.button1, length = self:SetupButtonVertical("PortalMageButton1", self.button1, length, "stormwind", 121, frame)
		end
		if portalMageSpells["ironforge"] then
			self.button2, length = self:SetupButtonVertical("PortalMageButton2", self.button2, length, "ironforge", 122, frame)
		end
		if portalMageSpells["darnassus"] then
			self.button3, length = self:SetupButtonVertical("PortalMageButton3", self.button3, length, "darnassus", 123, frame)
		end
	else
		if portalMageSpells["orgrimmar"] then
			self.button1, length = self:SetupButtonVertical("PortalMageButton1", self.button1, length, "orgrimmar", 121, frame)
		end
		if portalMageSpells["undercity"] then
			self.button2, length = self:SetupButtonVertical("PortalMageButton2", self.button2, length, "undercity", 122, frame)
		end
		if portalMageSpells["thunderbluff"] then
			self.button3, length = self:SetupButtonVertical("PortalMageButton3", self.button3, length, "thunderbluff", 123, frame)
		end
	end
	frame:SetWidth(30)
	move:SetWidth(30)
	frame:SetHeight(-length)
	move:SetHeight(-length)
	frame.teleports:ClearAllPoints()
	frame.portals:ClearAllPoints()
	frame.teleports:SetPoint("TOPLEFT", -15, 0)
	frame.portals:SetPoint("BOTTOMLEFT", -15, 0)
	collectgarbage("collect")
end

function PortalMage:SetupButtonsHorizontal(frame, move)
	self:ClearButtons()
	local length = 0
	if self.faction == "Alliance" then
		if portalMageSpells["stormwind"] then
			self.button1, length = self:SetupButtonHorizontal("PortalMageButton1", self.button1, length, "stormwind", 121, frame)
		end
		if portalMageSpells["ironforge"] then
			self.button2, length = self:SetupButtonHorizontal("PortalMageButton2", self.button2, length, "ironforge", 122, frame)
		end
		if portalMageSpells["darnassus"] then
			self.button3, length = self:SetupButtonHorizontal("PortalMageButton3", self.button3, length, "darnassus", 123, frame)
		end
	else
		if portalMageSpells["orgrimmar"] then
			self.button1, length = self:SetupButtonHorizontal("PortalMageButton1", self.button1, length, "orgrimmar", 121, frame)
		end
		if portalMageSpells["undercity"] then
			self.button2, length = self:SetupButtonHorizontal("PortalMageButton2", self.button2, length, "undercity", 122, frame)
		end
		if portalMageSpells["thunderbluff"] then
			self.button3, length = self:SetupButtonHorizontal("PortalMageButton3", self.button3, length, "thunderbluff", 123, frame)
		end
	end
	frame:SetWidth(length)
	move:SetWidth(length)
	frame:SetHeight(40)
	move:SetHeight(40)
	frame.teleports:ClearAllPoints()
	frame.portals:ClearAllPoints()
	frame.teleports:SetPoint("TOPLEFT", -15, 0)
	frame.portals:SetPoint("TOPRIGHT", 15, 0)
	collectgarbage("collect")
end

function PortalMage.buttonOnUpdate(self)
	if self == PortalMage.button1 then
		if PortalMage.faction == "Alliance" then
			_G[self:GetName().."Icon"]:SetTexture(PortalMage.icons["stormwind"])
		elseif PortalMage.faction == "Horde" then
			_G[self:GetName().."Icon"]:SetTexture(PortalMage.icons["orgrimmar"])
		end
	end
	if self == PortalMage.button2 then
		if PortalMage.faction == "Alliance" then
			_G[self:GetName().."Icon"]:SetTexture(PortalMage.icons["ironforge"])
		elseif PortalMage.faction == "Horde" then
			_G[self:GetName().."Icon"]:SetTexture(PortalMage.icons["undercity"])
		end
	end
	if self == PortalMage.button3 then
		if PortalMage.faction == "Alliance" then
			_G[self:GetName().."Icon"]:SetTexture(PortalMage.icons["darnassus"])
		elseif PortalMage.faction == "Horde" then
			_G[self:GetName().."Icon"]:SetTexture(PortalMage.icons["thunderbluff"])
		end
	end
	_G[self:GetName().."Icon"]:Show()
	if Masque ~= nil then
		group:ReSkin()
	end
end

function PortalMage:SetupButtonVertical(name, button, y, index, id, frame)
	if button == nil then
		button = CreateFrame("CHECKBUTTON", name, frame, "SecureActionButtonTemplate, ActionBarButtonTemplate")
	end
	button:Show()
	button:SetWidth(38)
	button:SetHeight(38)
	button:SetPoint("TOPLEFT", 0, y)
	button:SetFrameStrata("MEDIUM")
	button:SetAttribute("showgrid", 1)
	button:SetAttribute("action", id)
	button:SetAttribute("type", "spell")
	button:SetAttribute("spell1", self.portals[index])
	button:SetAttribute("spell2", self.teleports[index])
	button:SetScript("OnUpdate", self.buttonOnUpdate)
	button:SetScript("OnEnter", function(self) 
					 GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
					 GameTooltip:AddLine("Left Click:")
					 GameTooltip:AddSpellByID(self:GetAttribute("spell1"))
					 GameTooltip:AddLine("\n")
					 GameTooltip:AddLine("Right Click:")
					 GameTooltip:AddSpellByID(self:GetAttribute("spell2"))
					 GameTooltip:Show()
					end)
	button:SetScript("OnLeave", function(self) 
					 GameTooltip:Hide()
					end)
	return button, y - 40
end

function PortalMage:SetupButtonHorizontal(name, button, x, index, id, frame)
	if button == nil then
		button = CreateFrame("CHECKBUTTON", name, frame, "SecureActionButtonTemplate, ActionBarButtonTemplate")
	end
	button:Show()
	button:SetWidth(38)
	button:SetHeight(38)
	button:SetPoint("TOPLEFT", x, 0)
	button:SetFrameStrata("MEDIUM")
	button:SetAttribute("showgrid", 1)
	button:SetAttribute("action", id)
	button:SetAttribute("type", "spell")
	button:SetAttribute("spell1", self.portals[index])
	button:SetAttribute("spell2", self.teleports[index])
	button:SetScript("OnUpdate", self.buttonOnUpdate)
	button:SetScript("OnEnter", function(self) 
					 GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
					 GameTooltip:AddLine("Left Click:")
					 GameTooltip:AddSpellByID(self:GetAttribute("spell1"))
					 GameTooltip:AddLine("\n")
					 GameTooltip:AddLine("Right Click:")
					 GameTooltip:AddSpellByID(self:GetAttribute("spell2"))
					 GameTooltip:Show()
					end)
	button:SetScript("OnLeave", function(self) 
					 GameTooltip:Hide()
					end)
	return button, x + 40
end

function PortalMage:ClearButtons()
	self:ClearButton(self.button1, 121)
	self:ClearButton(self.button2, 122)
	self:ClearButton(self.button3, 123)
end

function PortalMage:ClearButton(button, id)
	if button ~= nil then
		button:Hide()
		button:SetAttribute("showgrid", 0)
		button:SetAttribute("action", id)
		button:SetAttribute("type", "spell")
		PickupAction(id)
		ClearCursor()
	end
end